from typing import Any
import streamlit as st


class LaunchingProfiler:
    def __init__(self, skip=False):
        if not skip:
            from pyinstrument import Profiler

            self.profiler = Profiler()
        else:
            self.profiler = None

    def __enter__(self):
        if self.profiler:
            import inspect

            self.profiler.start(caller_frame=inspect.currentframe().f_back)
        return self

    def __exit__(self, *args: Any):
        if self.profiler:
            from streamlit_embeded import st_embeded

            self.profiler.stop()
            with st.expander("System Profiling"):
                st_embeded(self.profiler.output_html(timeline=False), height=1000)
        return False



from st_click_detector import click_detector


def click_detector_wrapper(rank: str, svg: str, label: str):
    content = f"""
        <a href='#' id='rank-{label}-{rank}'><span style="color: rgba(0, 0, 0, 0.45); margin-right: 10px">{svg}</span>{label}</a>
    """
    return click_detector(html_content=content, key=f"{rank}-{label}-detector")



from pathlib import Path
from typing import List, Optional, Union, Dict
from pydantic import BaseModel
import json
from wcmatch import glob


class MetricsChartReportElement(BaseModel):
    metrics_path: Union[str, Path]
    options: Dict
    title: Optional[str] = ""
    type: str = "metrics_chart"
    description: str = ""
    height: str = "240px"


class DockingReportElement(BaseModel):
    protein_path: Union[str, Path]
    ligand_path: Union[str, Path]
    gt_ligand_path: Optional[Union[str, Path]]
    title: str = ""
    description: str = ""
    traj_path: Optional[Union[str, Path]] = None
    type: str = "docking"
    height: str = "240px"


class AutoReportElement(BaseModel):
    path: Union[str, Path, List[Union[str, Path]]]
    title: str = ""
    type: str = "auto"
    description: str = ""
    height: str = "240px"
    render_hints: Optional[List[str]] = None
    kwargs: Optional[Dict] = None

    @classmethod
    def from_patterns(cls, output_dir, patterns, **kwargs):
        return [
            cls(path=i, **kwargs)
            for i in glob.glob(root_dir=output_dir, patterns=patterns)
        ]


class ChartsReportElement(BaseModel):
    step: str
    ncols: int
    charts: List[Dict]


class ChartReportElement(BaseModel):
    options: Dict
    title: Optional[str] = ""
    type: str = "chart"
    description: str = ""


class StepChartsReportElement(BaseModel):
    steps: List[ChartsReportElement]
    title: Optional[str] = ""
    type: str = "step_charts"
    description: str = ""


class ReportSection(BaseModel):
    title: Optional[str] = ""
    elements: List[
        Union[
            DockingReportElement,
            AutoReportElement,
            StepChartsReportElement,
            MetricsChartReportElement,
            ChartReportElement,
        ]
    ]
    ncols: int = 1
    description: str = ""
    chemical_formula: Optional[List[str]] = []
    smiles: Optional[List[str]] = []
    tags: Optional[List[str]] = []
    metrics_group: str = "default"
    metrics: Optional[Dict[str, Union[str, int, float]]] = {}
    hyperparameters: Optional[Dict[str, Union[str, int, float]]] = {}
    show_overview: bool = False


class Report(BaseModel):
    title: Optional[str] = ""
    sections: List[ReportSection]
    name: str = "index"
    description: Optional[str] = ""

    def save(self, output_dir: Optional[Union[str, Path]] = ""):
        with (Path(output_dir) / f"{self.name}.launching_report").open("w") as f:
            f.write(self.json(indent=2))

    @classmethod
    def load(cls, path: Union[str, Path]):
        with Path(path).open() as f:
            report = json.load(f)
        return cls.parse_obj(report)


if __name__ == "__main__":
    ...



def lazy_property(func):
    attr_name = "_lazy_" + func.__name__

    @property
    def _lazy_property(self):
        if not hasattr(self, attr_name):
            setattr(self, attr_name, func(self))
        return getattr(self, attr_name)

    return _lazy_property



import os
import sh
import json
import uuid
import requests
from typing import Optional, Dict, Tuple, List, Any
from urllib.parse import urljoin
from loguru import logger
from pathlib import Path
from dp.launching.app import config
from dp.launching.utils.tools import lazy_property


class PaymentInfo:
    paid_account: str
    paid_project_id: str

    def __init__(self, paid_account, paid_project_id):
        self.paid_account = paid_account
        self.paid_project_id = paid_project_id


def ensure_path_exists(path: Path) -> Path:
    path.mkdir(parents=True, exist_ok=True)
    return path


class App:
    app_name: str
    app_secret: str

    def __init__(self, app_name, app_secret):
        self.app_name = app_name
        self.app_secret = app_secret

    @lazy_property
    def get_draft_job_root(self):
        path = config.LAUNCHING_ROOT / self.app_name / "draft-jobs"
        return ensure_path_exists(path)

    @lazy_property
    def get_job_root(self):
        path = config.LAUNCHING_ROOT / self.app_name / "jobs"
        return ensure_path_exists(path)

    def get_job_output_path(self, job_id: str):
        path = self.get_job_root / job_id / "outputs"
        return ensure_path_exists(path)

    def get_job_log_path(self, job_id: str):
        return self.get_job_output_path(job_id) / "STDOUTERR.log"

    @lazy_property
    def get_tmp_root(self):
        path = config.LAUNCHING_ROOT / self.app_name / "uploaded-files"
        return ensure_path_exists(path)

    @lazy_property
    def get_log_path(self):
        return config.LAUNCHING_ROOT / self.app_name / "jobs"

    def submit_job(
        self,
        payment_info: PaymentInfo,
        job_name: str = "",
        files: Dict = {},
        params: Dict = {},
        props: Dict = {},
        version: str = "",
        session_id: str = "",
        submit_params: Dict = {},
    ) -> Optional[str]:
        # TODO check user shortTerm ticket/orderId
        logger.info(f"submit job {job_name}: params {params}, props {props}")
        if config.DRY_RUN_MODE:
            return "fake-dryrun-job-id" + str(uuid.uuid4())

        creator = payment_info.paid_account
        job_id = f"{job_name}-{uuid.uuid4().hex[:8]}"
        draft_job_path = self.get_draft_job_root

        logger.info(f"start prepare inputs for job {job_id}")
        for field_name, origin_file_path in files.items():
            file_path = draft_job_path / field_name
            file_path.mkdir(parents=True, exist_ok=True)
            logger.debug(f"copy {origin_file_path} to {file_path}")
            sh.cp(
                origin_file_path, (file_path / os.path.basename(origin_file_path))
            )  # noqa
        logger.debug(f"end prepare inputs for job {job_id}")
        url = urljoin(config.LAUNCHING_API_BASE, "/api/jobs/")

        props = props or {}
        props.update(
            {
                "paid_account": payment_info.paid_account,
                "paid_project_id": payment_info.paid_project_id,
                # TODO support more login type
                "login_type": "bohrium",
            }
        )
        if session_id:
            props["session"] = session_id

        payload = json.dumps(
            {
                "job_id": job_id,
                "description": "job created from workbench",
                "application": self.app_name,
                "token": self.app_secret,
                "version": version,
                "creator": creator,
                "from_labs": True,  # TODO FIXME both launching & labs can access workbench
                "params": params,
                "properties": props,
                "submit_params": submit_params,
            }
        )
        headers = {"Content-Type": "application/json"}
        response = requests.request("POST", url, headers=headers, data=payload)
        try:
            response.raise_for_status()
        except Exception as e:
            logger.error(
                f"submit job {job_id} to launching failed, status code: {response.status_code}"
            )  # noqa
            logger.exception(e)
            return None
        logger.info(
            f"submit job {job_id} to launching success, status code: {response.status_code}"
        )  # noqa
        data = response.json()
        if data["code"] != 200:
            logger.error(data["error"])
            return None
        return job_id

    def get_job_status(self, payment_info: PaymentInfo, job_id: str):
        if config.DRY_RUN_MODE:
            return "success"

        url = urljoin(
            config.LAUNCHING_API_BASE, f"/api/jobs/{self.app_name}/{job_id}/status"
        )
        headers = {"Content-Type": "application/json"}
        response = requests.request("GET", url, headers=headers)
        try:
            response.raise_for_status()
        except Exception as e:
            logger.error(
                f"get job {job_id} status failed, status code: {response.status_code}"
            )  # noqa
            logger.exception(e)
            return "unknown"
        data = response.json()
        return data["status"]

    def get_job_logs(
        self, payment_info: PaymentInfo, job_id: str, nlast: int = 1000
    ) -> Optional[str]:
        if config.DRY_RUN_MODE:
            return "fake-dryrun-log"
        log_path = self.get_job_log_path(job_id)
        if log_path.exists():
            return str(sh.tail("-n", nlast, log_path))

    def cancel_job(self, payment_info: PaymentInfo, job_id: str) -> Optional[str]:
        payload = json.dumps(
            {
                "job_id": job_id,
                "application": self.app_name,
                "token": self.app_secret,
            }
        )
        headers = {"Content-Type": "application/json"}
        url = urljoin(config.LAUNCHING_API_BASE, "/api/jobs/stop_job")
        response = requests.request("POST", url, headers=headers, json=payload)
        try:
            response.raise_for_status()
        except Exception as e:
            logger.error(
                f"cancel job {job_id} failed, status code: {response.status_code}"
            )  # noqa
            logger.exception(e)
            return None
        logger.info(
            f"cancel job {job_id} success, status code: {response.status_code}"
        )  # noqa
        data = response.json()
        if data["code"] != 200:
            logger.error(data["error"])
            return None
        return job_id

    def get_jobs_by_session_id(
        self, payment_info: PaymentInfo, session_id: str
    ) -> Tuple[int, List[Any]]:
        if config.DRY_RUN_MODE:
            return 0, []

        url = urljoin(config.LAUNCHING_API_BASE, f"/api/jobs/{self.app_name}")
        session_id = session_id
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.app_secret}",
        }
        params = {"session_id": session_id}
        response = requests.request("GET", url, headers=headers, params=params)
        try:
            response.raise_for_status()
        except Exception as e:
            logger.error(
                f"get jobs for session {session_id} failed, status code: {response.status_code} {e}"
            )  # noqa
        data = response.json()
        if data.get("code") != 200:
            logger.error(data)
            logger.exception(Exception(data.get("error")))
        return data.get("count"), data.get("data")



import argparse
import inspect
import json
import os
import sys
import random
import string
import typing as T
import traceback
from pathlib import Path
from typing import Callable as F

from dp.launching.typing.basic import BaseModel
from pydantic.error_wrappers import ErrorWrapper
from pydantic_cli import (
    EpilogueHandlerType,
    ExceptionHandlerType,
    M,
    PrologueHandlerType,
    SubParser,
    default_epilogue_handler,
    default_exception_handler,
    default_minimal_exception_handler,
    default_prologue_handler,
)
from pydantic_cli import run_sp_and_exit as origin_run_sp_and_exit, to_runner_sp
from pydantic_cli import to_runner as origin_to_runner

__all__ = [
    "to_runner",
    "default_minimal_exception_handler",
    "SubParser",
    "run_sp_and_exit",
]

START_API_SERVER_CONFIG = "--start-api-server"

VALIDATE_ONLY_RUNNER = lambda *_, **__: 0


SERVICE_START_CONFIG = "start-service"
SERVICE_API_PREFIX = "/api"
SERVICE_API_VERSION = "v1"
SERVICE_STATUS_ERROR = "error"
SERVICE_STATUS_SUCCESS = "success"


def traverse(key, _args, target_type, cb):
    if (
        isinstance(_args, list)
        or isinstance(_args, T.Tuple)
        or isinstance(_args, tuple)
        or isinstance(_args, T.List)
    ):
        for arg in _args:
            traverse(key, arg, target_type, cb)
    elif isinstance(_args, dict):
        for _key, arg in _args.items():
            traverse(_key, arg, target_type, cb)
    elif isinstance(_args, target_type):
        isinstance(cb, T.Callable) and cb(key, _args)


def print_extra_help(entry):
    print(
        f"""usage: {entry} [-h] [--gen_schema | --gen-schema] [-o OUTPUT]

optional arguments:
  -h, --help                          Show this help message and exit
  --gen_schema, --gen-schema          Generate a schema json file.
  {START_API_SERVER_CONFIG}                  Start launching api server.
  -o OUTPUT, --output OUTPUT          Schema json file output path.
    """
    )


def get_schema_properties(model) -> dict:
    return model.schema(by_alias=True).get("properties", {})


def get_schema_references(model) -> dict:
    return model.schema(by_alias=True).get("definitions", {})


def get_required_properties(model) -> dict:
    return model.schema(by_alias=True).get("required", [])


def get_internal_meta(model) -> dict:
    return model.schema(by_alias=True).get("__internal_meta__", {})


def get_doc(model):
    try:
        doc = inspect.getdoc(model)
        if doc:
            return doc
        class_file = inspect.getfile(model)
        caller_path = Path(os.path.abspath(class_file))
        if not caller_path.exists():
            return ""
        scope = {}
        scope["__name__"] = ""
        with open(caller_path, "r") as f:
            exec(f.read(), scope, scope)
            doc = scope.get("__doc__", "")
            return doc
    except Exception as _:
        return ""


def get_response(code: int, status: str, msg: str):
    from fastapi.responses import JSONResponse

    return JSONResponse(
        content={
            "status": status,
            "msg": msg,
        },
        status_code=code,
    )


def get_random_str(len: int):
    return "".join(random.choices(string.ascii_letters + string.digits, k=len))


def get_api_v1(path: str):
    return f"{SERVICE_API_PREFIX}/{SERVICE_API_VERSION}/{path}"


def start_api_server(models, origin_kwargs=None):
    try:
        from fastapi import FastAPI, Request
        import uvicorn

        if os.getenv("PROD_ENV"):
            app = FastAPI(openapi_url=None)
        else:
            app = FastAPI()

        @app.post(get_api_v1("validate"))
        async def validate(request: Request):
            data = await request.json()

            form = data.get("form")
            sub_model = data.get("sub_model")
            form_json = None
            tmp_file_path = None
            if form:
                try:
                    form_json = json.dumps(form, indent=2, ensure_ascii=False)
                except Exception as err:
                    return get_response(400, SERVICE_STATUS_ERROR, err)

                tmp_file_path = Path(f"/tmp/{get_random_str(16)}.json")
                tmp_file_path.write_text(form_json)

            if not tmp_file_path:
                return get_response(
                    400, SERVICE_STATUS_ERROR, "request missed form data"
                )

            errors = {}
            try:
                ERROR_TYPE_ARG_ERR = "ARG_ERR"
                ERROR_TYPE_OTHER_ERR = "OTHER_ERR"

                def error_handler(ex: BaseException) -> int:
                    exc_type, exc_value, exc_traceback = sys.exc_info()
                    if hasattr(exc_value, "message"):
                        errors["error_type"] = ERROR_TYPE_OTHER_ERR
                        errors["message"] = exc_value.message
                    else:
                        errors["error_type"] = ERROR_TYPE_ARG_ERR
                        errors["message"] = str(exc_value)
                        if type(exc_value.args) in (T.Tuple, T.List, tuple, list):

                            def collect_err(key, value: ErrorWrapper):
                                if "errors" not in errors:
                                    errors["errors"] = {}
                                if value._loc not in errors["errors"]:
                                    errors["errors"][value._loc] = [str(value.exc)]
                                else:
                                    errors["errors"][value._loc].append(str(value.exc))

                            traverse("", exc_value.args, ErrorWrapper, collect_err)
                    return 1

                if "exception_handler" in origin_kwargs:
                    del origin_kwargs["exception_handler"]

                if type(models) == dict:
                    f = to_runner_sp(
                        **origin_kwargs,
                        subparsers=models,
                        exception_handler=error_handler,
                    )
                    f([sub_model, "--json-config", str(tmp_file_path)])

                else:
                    origin_to_runner(
                        **origin_kwargs,
                        cls=models,
                        exception_handler=error_handler,
                    )(["--json-config", str(tmp_file_path)])
                if errors:
                    return get_response(500, SERVICE_STATUS_ERROR, errors)
                else:
                    return get_response(200, SERVICE_STATUS_SUCCESS, None)
            except Exception as err:
                return get_response(500, SERVICE_STATUS_ERROR, errors)

        uvicorn.run(app, host="0.0.0.0", port=3190)

    except Exception as err:
        raise Exception(f"start launching sdk service falied: {err}")


def get_schema(model, type: str) -> dict:
    return {
        "model_type": type,
        "documentation": get_doc(model),
        "description": model.description if hasattr(model, "description") else "",
        "schema_properties": get_schema_properties(model),
        "schema_references": get_schema_references(model),
        "required_properties": get_required_properties(model),
        "__internal_meta__": get_internal_meta(model),
    }


def get_internal_schemas_output_path(name: str, output_path: str):
    path = Path(output_path)
    path.mkdir(exist_ok=True, parents=True)
    path = path / (name + ".json")
    return path


def gen_model_schema(model, output_path, name=None, type=""):
    res = get_schema(model, type)
    path = get_internal_schemas_output_path(name or model.__name__, output_path)
    path.write_text(json.dumps(res, indent=2, ensure_ascii=False))


class to_runner(origin_to_runner):
    def __init__(
        self,
        cls: T.Type[M],
        runner_func: F[[M], int],
        description: T.Optional[str] = None,
        version: T.Optional[str] = None,
        exception_handler: ExceptionHandlerType = default_exception_handler,
        prologue_handler: PrologueHandlerType = default_prologue_handler,
        epilogue_handler: EpilogueHandlerType = default_epilogue_handler,
    ):
        self.model = cls
        self.runner_func = runner_func
        self.description = description
        self.version = version
        super().__init__(
            cls,
            runner_func,
            description,
            version,
            exception_handler,
            prologue_handler,
            epilogue_handler,
        )

    def __call__(self, args: T.List[str]) -> int:
        if len(sys.argv) == 1:
            print_extra_help(sys.argv[0])
        for item in args:
            if item == "-h" or item == "--help":
                print_extra_help(sys.argv[0])
                return super().__call__(args)
            elif item == "--gen_schema":
                parser = argparse.ArgumentParser("Launching-Schema-Gen")
                parser.add_argument(
                    "--gen_schema",
                    action="store_true",
                    default=False,
                    help="Generate a schema json file.",
                )
                parser.add_argument(
                    "-o",
                    "--output",
                    type=str,
                    default="generated_schemas",
                    help="Schema json file output path.",
                )
                gen_schema_args = parser.parse_args()
                return self.gen_schema(gen_schema_args.output)
            elif item == "--gen-schema":
                parser = argparse.ArgumentParser("Launching-Schema-Gen")
                parser.add_argument(
                    "--gen-schema",
                    action="store_true",
                    default=False,
                    help="Generate a schema json file.",
                )
                parser.add_argument(
                    "-o",
                    "--output",
                    type=str,
                    default="generated_schemas",
                    help="Schema json file output path.",
                )
                gen_schema_args = parser.parse_args()
                return self.gen_schema(gen_schema_args.output)
            elif item == START_API_SERVER_CONFIG:
                self.runner_func = VALIDATE_ONLY_RUNNER
                return start_api_server(
                    self.model,
                    {
                        "runner_func": self.runner_func,
                        "description": self.description,
                        "version": self.version,
                    },
                )

        return sys.exit(super().__call__(args))

    def gen_schema(self, output_path):
        try:
            hasattr(self, "model") and gen_model_schema(
                self.model, output_path, None, "single"
            )
            print(
                f"JSONSchema describe file for {self.model.__name__} has been generated successfully to {output_path}/{self.model.__name__}.json"
            )
            print(
                f"Verify your schema at Dev Assistant https://launching.mlops.dp.tech/?request=GET%3A%2Fdeveloper_assistant"
            )
        except Exception as err:
            import traceback

            traceback.print_exc()
            print("gen schema failed: ", err)


class run_sp_and_exit:
    def __init__(self, *args, **kwargs) -> None:
        if len(sys.argv) == 1:
            print_extra_help(sys.argv[0])
        if len(sys.argv) >= 1:
            for item in sys.argv[1:]:
                if item == "-h" or item == "--help":
                    print_extra_help(sys.argv[0])
                elif item == "--gen_schema":
                    parser = argparse.ArgumentParser("Launching-Schema-Gen")
                    parser.add_argument(
                        "--gen_schema",
                        action="store_true",
                        default=False,
                        help="Generate a schema json file. Default to ./generated_schemas/",
                    )
                    parser.add_argument(
                        "-o",
                        "--output",
                        type=str,
                        default="generated_schemas",
                        help="Schema json file output path.",
                    )
                    gen_schema_args = parser.parse_args()
                    self.gen_schema(gen_schema_args.output, args, kwargs)
                    return
                elif item == "--gen-schema":
                    parser = argparse.ArgumentParser("Launching-Schema-Gen")
                    parser.add_argument(
                        "--gen-schema",
                        action="store_true",
                        default=False,
                        help="Generate a schema json file.",
                    )
                    parser.add_argument(
                        "-o",
                        "--output",
                        type=str,
                        default="generated_schemas",
                        help="Schema json file output path.",
                    )
                    gen_schema_args = parser.parse_args()
                    self.gen_schema(gen_schema_args.output, args, kwargs)
                    return
                elif item == START_API_SERVER_CONFIG:
                    models = {}

                    def __update_runner_func(key, sub_parser):
                        if hasattr(sub_parser, "runner_func"):
                            models[key] = sub_parser
                            sub_parser.runner_func = VALIDATE_ONLY_RUNNER

                    traverse("", args, SubParser, __update_runner_func)
                    return start_api_server(models, kwargs)

        if "exception_handler" not in kwargs:
            kwargs["exception_handler"] = default_minimal_exception_handler
        origin_run_sp_and_exit(*args, **kwargs)

    def gen_schema(self, output, args, kwargs):
        self.models = self.__get_models({"tmp1": args, "tmp2": kwargs})
        try:
            for name, model in self.models.items():
                gen_model_schema(model, output, name, "multiple")
                print(
                    f"JSONSchema describe file for {model.__name__} has been generated successfully to {output}/{name}.json"
                )
            print(
                f"Verify your schema at Dev Assistant https://launching.mlops.dp.tech/?request=GET%3A%2Fdeveloper_assistant"
            )
        except Exception as err:
            print("gen schemas failed: ", err)

    def __get_models(self, origin: dict):
        res = {}
        for key, value in origin.items():
            if isinstance(value, dict):
                res.update(self.__get_models(value))
            elif isinstance(value, list) or isinstance(value, tuple):
                for i in value:
                    res.update(self.__get_models(i))
            elif isinstance(value, BaseModel):
                res.update({value.__name__: value})
            elif isinstance(value, SubParser):
                value.model_class.description = value.description or ""
                value.documentation = get_doc(value.model_class)
                res.update({key: value.model_class})
        return res



import os
from pathlib import Path
from tempfile import gettempdir

DEBUG_MODE = True if os.getenv("DEBUG_MODE") in ("True", "true", "1") else False

DRY_RUN_MODE = True if os.getenv("DRY_RUN_MODE") in ("True", "true", "1") else False

LAUNCHING_API_BASE = os.getenv("LAUNCHING_API_BASE", "https://launching.local.dp.tech")

LAUNCHING_ROOT = Path(os.getenv("LAUNCHING_ROOT") or "var").absolute()
LAUNCHING_ROOT.mkdir(parents=True, exist_ok=True)

# LAUNCHING_JOBS_ROOT = LAUNCHING_ROOT / "jobs"
# LAUNCHING_JOBS_ROOT.mkdir(parents=True, exist_ok=True)
# LAUNCHING_JOBS_DRAFT_ROOT = LAUNCHING_ROOT / "draft-jobs"
# LAUNCHING_JOBS_DRAFT_ROOT.mkdir(parents=True, exist_ok=True)

LAUNCHING_UPLOADED_FILE_TMPDIR = Path(gettempdir()) / "uploaded-files"
LAUNCHING_UPLOADED_FILE_TMPDIR.mkdir(parents=True, exist_ok=True)

FAKE_USER = os.getenv("FAKE_USER", "fake@dp.tech")



import uuid
import os
import os.path
import json
from pathlib import Path
from urllib.parse import urljoin
from typing import Optional, Dict, Tuple, Any, List

import sh
import requests
from loguru import logger
import streamlit as st

from dp.launching.workbench.config import (
    DRY_RUN_MODE,
    LAUNCHING_API_BASE,
    LAUNCHING_APPLICATION_NAME,
    LAUNCHING_APPLICATION_VERSION,
    LAUNCHING_APPLICATION_OWNER,
    LAUNCHING_APPLICATION_TOKEN,
    LAUNCHING_JOBS_ROOT,
    LAUNCHING_JOBS_DRAFT_ROOT,
)

from dp.launching.workbench.user import (
    get_current_user,
    get_current_session,
)

__all__ = ["get_session_saved_data", "save_session_data"]

LAUNCHING_SESSION_KEY = "launching_saved_data"


def get_session_saved_data(key, default=""):
    data = _get_session_saved_data()
    if not data:
        return default
    return data.get(key, default)


def _get_session_saved_data():
    data = st.session_state.get(LAUNCHING_SESSION_KEY)

    if not data:
        if DRY_RUN_MODE:
            return {
                "target_molecule": "CCO",
                "options": {
                    "retro_system": "unimol0621_bart0702_similarity",
                    "check_system": "unimol0717",
                    "optional_retro_system": "",
                },
                "current_step": 2,
            }
        data = _get_session_saved_data_from_remote() or {}
        st.session_state[LAUNCHING_SESSION_KEY] = data
    return data


def _get_session_saved_data_from_remote():
    session_id = get_current_session()
    params = {
        "session_id": session_id,
        "user": get_current_user(),
    }
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {LAUNCHING_APPLICATION_TOKEN}",
    }
    url = urljoin(
        LAUNCHING_API_BASE,
        f"/api/jobs/{LAUNCHING_APPLICATION_NAME}/session/{session_id}",
    )
    response = requests.request("GET", url, headers=headers, params=params)
    try:
        response.raise_for_status()
    except Exception as e:
        logger.error(
            f"get session data {session_id} failed, status code: {response.status_code} {e}"
        )  # noqa
    data = response.json()
    if data.get("code") != 200:
        logger.error(data)
        logger.exception(Exception(data.get("error")))
    return data.get("data")


def save_session_data(data: dict):
    if os.getenv("DRY_RUN_MODE"):
        session_data = _get_session_saved_data()
        session_data.update(data)
        st.session_state[LAUNCHING_SESSION_KEY] = session_data
        return data

    session_id = get_current_session()
    req_data = {
        "session_id": int(session_id),
        "application_name": LAUNCHING_APPLICATION_NAME,
        "application_token": LAUNCHING_APPLICATION_TOKEN,
        "user": get_current_user(),
        "data": data,
    }
    url = urljoin(
        LAUNCHING_API_BASE,
        f"/api/jobs/{LAUNCHING_APPLICATION_NAME}/session/{session_id}",
    )
    headers = {"Content-Type": "application/json"}
    response = requests.request("POST", url, headers=headers, json=req_data)
    try:
        response.raise_for_status()
    except Exception as e:
        logger.error(
            f"save session {session_id} failed, status code: {response.status_code} {e}"
        )  # noqa
    resp = response.json()
    if resp.get("code") != 200:
        logger.error(data)
        logger.exception(Exception(resp.get("error")))

    # save to session state
    session_data = _get_session_saved_data()
    session_data.update(data)
    st.session_state[LAUNCHING_SESSION_KEY] = session_data

    return session_data



import os

import jwt
import base64
from pathlib import Path

from loguru import logger
import streamlit as st
import extra_streamlit_components as stx

from dp.launching.workbench.config import (
    LAUNCHING_SESSIONS_ROOT,
    FAKE_USER,
    WORKSPACE_INFO_SESSION_KEY,
    LAUNCHING_APPLICATION_VERSION,
)


LAUNCHING_JWT_PUBLIC_KEY = os.getenv("LAUNCHING_JWT_PUBLIC_KEY")

cookie_manager = stx.CookieManager()

__all__ = [
    "get_current_session",
    "get_current_session_name",
    "get_current_user",
    "get_current_paid_account",
    "get_current_bohrium_project",
    "get_user_session_root",
    "is_anonymous_user",
]


def get_current_session():
    if os.getenv("DRY_RUN_MODE"):
        return 1
    info = _get_current_workbench_info()
    return info["payload"]["session"]


def get_current_session_name():
    if os.getenv("DRY_RUN_MODE"):
        return "fake-session"
    info = _get_current_workbench_info()
    return info["payload"]["session-name"]


def get_current_user():
    if os.getenv("DRY_RUN_MODE"):
        return FAKE_USER
    info = _get_current_workbench_info()
    return info["name"] if info else None


def get_current_bohrium_project_available():
    if os.getenv("DRY_RUN_MODE"):
        return True
    info = _get_current_workbench_info()
    return info["payload"]["paid_project_available"]


def get_current_bohrium_project():
    if os.getenv("DRY_RUN_MODE"):
        return "fake-project"
    info = _get_current_workbench_info()
    return info["payload"]["paid_project_id"]


def get_current_paid_account():
    if os.getenv("DRY_RUN_MODE"):
        return "fake-account"
    info = _get_current_workbench_info()
    return info["payload"]["paid_account"]


def get_current_job_version():
    if os.getenv("DRY_RUN_MODE"):
        return "fake-project"
    info = _get_current_workbench_info()
    return info["payload"].get("job_version", LAUNCHING_APPLICATION_VERSION)


def _get_current_workbench_info():
    if st.session_state.get(WORKSPACE_INFO_SESSION_KEY):
        return st.session_state.get(WORKSPACE_INFO_SESSION_KEY)
    query_string = st.experimental_get_query_params()
    token = query_string.get("dp_token")
    if not token:
        return None
    info = None
    try:
        public_key = base64.b64decode(LAUNCHING_JWT_PUBLIC_KEY)
        info = jwt.decode(
            jwt=token[0],
            key=public_key,
            algorithms=[
                "RS256",
            ],
        )
    except Exception as e:
        logger.error(f"failed get_session_from_token: {e}")
    if info:
        st.session_state[WORKSPACE_INFO_SESSION_KEY] = info
    return info


def get_user_session_root(user: str) -> Path:
    root = LAUNCHING_SESSIONS_ROOT / user
    if not root.exists():
        root.mkdir(parents=True, exist_ok=True)
    return root


def is_anonymous_user(user: str) -> bool:
    return user == "anonymous@anonymous.tech"



import os
from pathlib import Path
from tempfile import gettempdir

NOT_SYNCED_PREFIX = "$NOT_SYNCED$"

DEBUG_MODE = True if os.getenv("DEBUG_MODE") in ("True", "true", "1") else False

DRY_RUN_MODE = True if os.getenv("DRY_RUN_MODE") in ("True", "true", "1") else False

LAUNCHING_API_BASE = os.getenv("LAUNCHING_API_BASE", "https://launching.local.dp.tech")

LAUNCHING_APPLICATION_NAME = os.getenv("LAUNCHING_APPLICATION_NAME")
LAUNCHING_APPLICATION_TOKEN = os.getenv("LAUNCHING_APPLICATION_TOKEN")
LAUNCHING_APPLICATION_OWNER = os.getenv("LAUNCHING_APPLICATION_OWNER")
LAUNCHING_APPLICATION_VERSION = os.getenv("LAUNCHING_APPLICATION_VERSION")

LAUNCHING_ROOT = Path(os.getenv("LAUNCHING_ROOT") or "var").absolute()
LAUNCHING_ROOT.mkdir(parents=True, exist_ok=True)

LAUNCHING_SESSIONS_ROOT = (
    Path(os.getenv("LAUNCHING_SESSIONS_ROOT"))
    if os.getenv("LAUNCHING_SESSIONS_ROOT")
    else LAUNCHING_ROOT / "sessions"
)
LAUNCHING_SESSIONS_ROOT.mkdir(parents=True, exist_ok=True)
LAUNCHING_JOBS_ROOT = LAUNCHING_ROOT / "jobs"
LAUNCHING_JOBS_ROOT.mkdir(parents=True, exist_ok=True)
LAUNCHING_JOBS_DRAFT_ROOT = LAUNCHING_ROOT / "draft-jobs"
LAUNCHING_JOBS_DRAFT_ROOT.mkdir(parents=True, exist_ok=True)

LAUNCHING_UPLOADED_FILE_TMPDIR = Path(gettempdir()) / "uploaded-files"
LAUNCHING_UPLOADED_FILE_TMPDIR.mkdir(parents=True, exist_ok=True)

FAKE_USER = os.getenv("FAKE_USER", "fake@dp.tech")


def get_not_synced_key(user_key: str) -> str:
    return NOT_SYNCED_PREFIX + "_" + user_key


WORKSPACE_INFO_SESSION_KEY = get_not_synced_key("current_workspace_info")



from user_agents import parse

from streamlit.runtime import Runtime
from streamlit.runtime.scriptrunner import get_script_run_ctx


def _get_session_id():
    context = get_script_run_ctx()
    if not context:
        return
    return context.session_id


def _get_current_request():
    session_id = _get_session_id()
    if not session_id:
        return None
    runtime = Runtime._instance
    if not runtime:
        return
    client = runtime.get_client(session_id)
    if not client:
        return
    return client.request


def get_web_origin():
    request = _get_current_request()
    return request.headers["Origin"] if request else None


def get_request_headers():
    request = _get_current_request()
    return request.headers if request else None


def get_ua() -> str:
    headers = get_request_headers()
    if headers:
        return headers.get("User-Agent")
    return None


def is_on_mac_chrome():
    # Windows Chrome: Mozilla/5.0 (Windows NT 10.0; Win64; x64)       AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36
    # Mac Chrome:     Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36
    # Mac Safari:     Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.6.1 Safari/605.1.15
    ua = get_ua()
    return ua.find("Mac OS") > -1 and ua.find("Chrome") > -1


def get_remote_ip():
    headers = get_request_headers()
    if not headers:
        return
    client_ips = (
        headers.get("X-Original-Forwarded-For")
        or headers.get("X-Forwarded-For")
        or headers.get("X-Real-Ip")
    )
    if not client_ips:
        return
    return client_ips.split(",")[0]


def get_user_agent():
    ua_string = get_ua() or ""
    return parse(ua_string)


def is_iphone_access():
    ua = get_user_agent()
    return ua.is_mobile and ua.os.family == "iOS"


def is_mobile_access():
    user_agent = get_user_agent()
    return user_agent.is_mobile if user_agent else False



import streamlit as st

__all__ = ["step_session_prefix", "reset_after_step_session", "clear_step_session"]


def step_session_prefix(step: int):
    return f"step-{step}-"


def reset_after_step_session(keep_step: int, max_step: int):
    for step in range(max_step, keep_step, -1):
        clear_step_session(step)


def clear_step_session(step: int):
    prefix = step_session_prefix(step)
    for key in st.session_state.keys():
        if key.startswith(prefix):
            del st.session_state[key]



import uuid
import os
import os.path
import json
from pathlib import Path
from urllib.parse import urljoin
from typing import Optional, Dict, Tuple, Any, List

import sh
import requests
from loguru import logger
import streamlit as st

from dp.launching.workbench.config import (
    DRY_RUN_MODE,
    LAUNCHING_API_BASE,
    LAUNCHING_APPLICATION_NAME,
    LAUNCHING_APPLICATION_VERSION,
    LAUNCHING_APPLICATION_OWNER,
    LAUNCHING_APPLICATION_TOKEN,
    LAUNCHING_JOBS_ROOT,
    LAUNCHING_JOBS_DRAFT_ROOT,
)

from dp.launching.workbench.user import (
    get_current_bohrium_project,
    get_current_paid_account,
)

from dp.launching.workbench.session import get_current_session


__all__ = ["submit_job", "get_job_status", "get_job_logs", "get_job_output_root"]


def submit_job(
        job_name: str,
        files: Dict,
        params: Dict,
        props: Dict,
        version: str = "",
) -> Optional[str]:
    logger.info(f"submit job {job_name}: params {params}, props {props}")

    if DRY_RUN_MODE:
        return "fake-dryrun-job-id" + str(uuid.uuid4())

    creator = getattr(st.session_state, "user", "")

    job_id = f"{job_name}-{uuid.uuid4().hex[:8]}"
    job_path = LAUNCHING_JOBS_DRAFT_ROOT / job_id / "inputs"
    job_path.mkdir(parents=True, exist_ok=True)

    logger.debug(f"start prepare inputs for job {job_id}")
    for field_name, origin_file_path in files.items():
        file_path = job_path / field_name
        file_path.mkdir(parents=True, exist_ok=True)
        logger.debug(f"copy {origin_file_path} to {file_path}")
        sh.cp(
            origin_file_path, (file_path / os.path.basename(origin_file_path))
        )  # noqa
    logger.debug(f"end prepare inputs for job {job_id}")

    url = urljoin(LAUNCHING_API_BASE, "/api/jobs/")

    props = props or {}
    props.update(
        {
            "paid_account": get_current_paid_account(),
            "paid_project_id": get_current_bohrium_project(),
            "session": int(get_current_session())
        }
    )

    # TODO support more login type
    props["login_type"] = "bohrium"
    payload = json.dumps(
        {
            "job_id": job_id,
            "description": "job created from workbench",
            "application": LAUNCHING_APPLICATION_NAME,
            "token": LAUNCHING_APPLICATION_TOKEN,
            "version": version or LAUNCHING_APPLICATION_VERSION,
            "creator": creator or LAUNCHING_APPLICATION_OWNER,
            "from_labs": True,  # FIXME both launching & labs can access workbench
            "params": params,
            "properties": props,
        }
    )
    headers = {"Content-Type": "application/json"}
    response = requests.request("POST", url, headers=headers, data=payload)
    try:
        response.raise_for_status()
    except Exception as e:
        logger.error(
            f"submit job {job_id} to launching failed, status code: {response.status_code}"
        )  # noqa
        logger.exception(e)
        return None
    logger.info(
        f"submit job {job_id} to launching success, status code: {response.status_code}"
    )  # noqa
    data = response.json()
    if data["code"] != 200:
        st.error(data["error"])
        logger.error(data["error"])
        return None
    return job_id


def get_job_status(job_id: str):
    if DRY_RUN_MODE:
        return "success"

    url = urljoin(
        LAUNCHING_API_BASE, f"/api/jobs/{LAUNCHING_APPLICATION_NAME}/{job_id}/status"
    )
    headers = {"Content-Type": "application/json"}
    response = requests.request("GET", url, headers=headers)
    try:
        response.raise_for_status()
    except Exception as e:
        logger.error(
            f"get job {job_id} status failed, status code: {response.status_code}"
        )  # noqa
        logger.exception(e)
        return "unknown"
    data = response.json()
    return data["status"]


def get_job_logs(job_id: str, nlast: int = 1000) -> Optional[str]:
    if DRY_RUN_MODE:
        return "fake-dryrun-logs"
    log_path = get_job_output_root(job_id) / "STDOUTERR.log"
    if log_path.exists():
        return str(sh.tail("-n", nlast, log_path))


def get_job_output_root(job_id: str) -> Path:
    return LAUNCHING_JOBS_ROOT / job_id / "outputs"


def cancel_job(job_id: str) -> Optional[str]:
    payload = json.dumps(
        {
            "job_id": job_id,
            "application": LAUNCHING_APPLICATION_NAME,
            "token": LAUNCHING_APPLICATION_TOKEN,
        }
    )
    headers = {"Content-Type": "application/json"}
    url = urljoin(LAUNCHING_API_BASE, "/api/jobs/stop_job")
    response = requests.request("POST", url, headers=headers, json=payload)
    try:
        response.raise_for_status()
    except Exception as e:
        logger.error(
            f"cancel job {job_id} failed, status code: {response.status_code}"
        )  # noqa
        logger.exception(e)
        return None
    logger.info(
        f"cancel job {job_id} success, status code: {response.status_code}"
    )  # noqa
    data = response.json()
    if data["code"] != 200:
        st.error(data["error"])
        logger.error(data["error"])
        return None
    return job_id


def get_jobs_by_session_id() -> Tuple[int, List[Any]]:
    if DRY_RUN_MODE:
        return 0, []

    url = urljoin(
        LAUNCHING_API_BASE, f"/api/jobs/{LAUNCHING_APPLICATION_NAME}"
    )
    session_id = get_current_session()
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {LAUNCHING_APPLICATION_TOKEN}"
    }
    params = {"session_id": session_id}
    response = requests.request("GET", url, headers=headers, params=params)
    try:
        response.raise_for_status()
    except Exception as e:
        logger.error(
            f"get jobs for session {session_id} failed, status code: {response.status_code} {e}"
        )  # noqa
    data = response.json()
    if data.get('code') != 200:
        logger.error(data)
        logger.exception(Exception(data.get('error')))
    return data.get('count'), data.get('data')



import streamlit as st
from dp.launching.workbench.utils.session import is_iphone_access

IPHONE_MAGIC_TYPES = {"pdb", "sdf", "mol", "pdbqt"}


def file_uploader(
    label,
    type=None,
    accept_multiple_files=False,
    key=None,
    help=None,
    on_change=None,
    args=None,
    kwargs=None,
    *,
    disabled=False,
    label_visibility="visible"
):
    if type and is_iphone_access():
        if isinstance(type, (list, tuple)) and any(
            i.lower() in IPHONE_MAGIC_TYPES for i in type
        ):
            type = None
        elif isinstance(type, str) and type.lower() in IPHONE_MAGIC_TYPES:
            type = None
    return st.file_uploader(
        label,
        type,
        accept_multiple_files,
        key,
        help,
        on_change,
        args,
        kwargs,
        disabled=disabled,
        label_visibility=label_visibility,
    )



import os
import zipfile

__all__ = ["zip_directory"]


def zip_directory(folder_path, zip_path):
    zipf = zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED)
    for root, dirs, files in os.walk(folder_path):
        for file in files:
            rel_path = os.path.relpath(os.path.join(root, file), folder_path)
            zipf.write(os.path.join(root, file), rel_path)
    zipf.close()



import base64
from typing import Union, Optional, List, Dict, Tuple, Set, Any, Literal
from datetime import datetime, date
from enum import Enum

from pydantic import (
    BaseModel as BBaseModel,
    Field,
    ValidationError,  # noqa
    parse_obj_as,  # noqa
    json,  # noqa
    parse_raw_as,  # noqa
    validator,  # noqa
    root_validator,  # noqa
)
from pydantic.fields import FieldInfo, ModelField
from pydantic_cli import DefaultConfig
from pydantic.main import model_schema

__all__ = [
    "BaseModel",
    "Field",
    "parse_obj_as",
    "ValidationError",
    "json",
    "parse_raw_as",
    "Union",
    "Optional",
    "List",
    "Dict",
    "Tuple",
    "Set",
    "Enum",
    "Literal",
    "Int",
    "String",
    "Float",
    "DateTime",
    "Date",
    "Boolean",
    "GenUUID",
    "MetadataToken",
    "validator",
    "root_validator",
    "MinMaxRange",
]

Int = int
String = str
Float = float
DateTime = datetime
Boolean = bool
Date = date

# __INTERNAL_KEY__ = 'TYPING_INTERNAL_PREFIX_' + \
#     ''.join(random.choices(string.ascii_letters + string.digits, k=10))

__INTERNAL_KEY__ = "TYPING_INTERNAL_PREFIX_TYPING"


def get_internal_key():
    return __INTERNAL_KEY__


class BaseModel(BBaseModel):
    # __id = None
    __fields = None
    __internal_meta = {}
    __hooks = {"field_info_extra": []}

    @classmethod
    def __get_id__(cls):
        return cls.__qualname__ or cls.__name__

    @classmethod
    def __add_field_info_extra_hook__(cls, hook):
        if isinstance(hook, function):
            cls.__hooks["field_info_extra"].append(hook)

    @classmethod
    def __add_meta_id__(cls):
        for field in cls.__fields__.values():
            if isinstance(field, ModelField):
                field.field_info.extra = {
                    **field.field_info.extra,
                    "__current": f"field_{field.name}",
                    "__parent": cls.__get_id__(),
                }
        return cls.__fields__

    @classmethod
    def __get_fields__(cls):
        if not cls.__fields:
            cls.__fields = cls.__add_meta_id__()
        return cls.__fields

    @classmethod
    def __get_schema_json__(cls, schema):
        import json
        from json import JSONEncoder

        class CustomEncoder(JSONEncoder):
            def setEncoder(self, obj):
                return list(obj)

            def default(self, obj):
                from datetime import date
                from datetime import datetime

                if isinstance(obj, datetime):
                    return obj.isoformat()
                elif isinstance(obj, date):
                    return obj.isoformat()
                elif isinstance(obj, set):
                    return self.setEncoder(obj)
                elif hasattr(obj, "__get_field_schema__"):
                    return obj.__get_field_schema__()
                return super().default(obj)

        return json.dumps(schema, cls=CustomEncoder, indent=2, ensure_ascii=False)

    @classmethod
    def schema(cls, *args, **kwargs):
        tmp_names = {}
        for base in [cls] + list(cls.__bases__):
            if isinstance(base, type) and issubclass(base, BaseModel):
                for field in base.__fields__.values():
                    if isinstance(field, ModelField):
                        # 这里不知道为啥, 如果直接用 field.field_info.extra.update 的方式去改
                        # 就会不生效
                        if "__current" not in field.field_info.extra:
                            tmp_names[field.name] = {"__current": f"field_{field.name}"}
                        if "__parent" not in field.field_info.extra:
                            tmp_names[field.name] = {
                                **tmp_names[field.name],
                                "__parent": base.__get_id__(),
                            }
                cls.__internal_meta[base.__get_id__()] = base.origin_schema()
                if hasattr(base, "__get_group_info__"):
                    cls.__internal_meta[base.__get_id__()][
                        "group"
                    ] = base.__get_group_info__()

        for field in cls.__fields__.values():
            if isinstance(field, ModelField):
                if "__current" not in field.field_info.extra:
                    field.field_info.extra.update(
                        {"__current": tmp_names[field.name]["__current"]}
                    )
                if "__parent" not in field.field_info.extra:
                    field.field_info.extra.update(
                        {"__parent": tmp_names[field.name]["__parent"]}
                    )

        res = model_schema(cls, *args, **kwargs)
        import json

        if get_internal_key() in res["properties"]:
            del res["properties"][get_internal_key()]
        res = json.loads(cls.__get_schema_json__(res))
        # res = json.loads(json.dumps(res))
        cls_internal_meta = cls.__internal_meta
        res["__internal_meta__"] = cls_internal_meta
        res = json.loads(cls.__get_schema_json__(res))
        return res

    @classmethod
    def origin_schema(cls, *args, **kwargs):
        return super().schema(*args, **kwargs)

    class Config(DefaultConfig):
        CLI_JSON_ENABLE = True


class GenUUID(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="gen_uuid", scope="executor")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "GenUUID":
        if isinstance(value, GenUUID):
            return value
        elif isinstance(value, str):
            return GenUUID(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return GenUUID(base64.b64encode(value).decode())
        else:
            raise Exception("Wrong type")


class MetadataToken(str):
    def get_value(self):
        return str(self)

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(format="metadata_token")

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "MetadataToken":
        if isinstance(value, MetadataToken):
            return value
        elif isinstance(value, str):
            return MetadataToken(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return MetadataToken(base64.b64encode(value).decode())
        else:
            raise Exception(f"MetadataToken Wrong type: {type(value)}")


class MinMaxRange(str):
    def get_value(self) -> Tuple[float, float]:
        _min, _max = map(float, str(self).split(","))
        return _min, _max

    @property
    def lower(self) -> float:
        return self.get_value()[0]

    @property
    def upper(self) -> float:
        return self.get_value()[1]

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(
            format="min_max_range",
            render_hint="float",
        )

    @classmethod
    def __get_validators__(cls) -> Any:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any) -> "MinMaxRange":
        if isinstance(value, MinMaxRange):
            return value
        elif isinstance(value, tuple) and len(value) == 2:
            return MinMaxRange(f"{value[0]},{value[1]}")
        elif isinstance(value, str):
            return MinMaxRange(value)
        elif isinstance(value, (bytes, bytearray, memoryview)):
            return MinMaxRange(base64.b64encode(value).decode())
        else:
            raise Exception(f"MinMaxRange Wrong type: {type(value)}")



import uuid
import os.path
from pathlib import Path

from loguru import logger
from streamlit.runtime.uploaded_file_manager import UploadedFile

from dp.launching.workbench.config import LAUNCHING_UPLOADED_FILE_TMPDIR

__all__ = ["save_upload_file"]


def save_upload_file(user: str, uploaded_file: UploadedFile):
    user_upload_path: Path = LAUNCHING_UPLOADED_FILE_TMPDIR / user
    user_upload_path.mkdir(parents=True, exist_ok=True)
    basename = os.path.basename(uploaded_file.name)
    fname, ext = tuple(os.path.splitext(basename))
    random_str = uuid.uuid4().hex
    fullpath = user_upload_path / f"{fname}-{random_str}{ext}"
    fullpath.write_bytes(uploaded_file.getvalue())
    logger.info(f"save_upload_file: {user}/{uploaded_file.name} -> {fullpath}")
    return fullpath



import setuptools

setuptools.setup(
    name="dp-launching-sdk",
    version="0.11.2",
    author="",
    author_email="",
    description="",
    long_description="",
    long_description_content_type="text/plain",
    url="",
    entry_points={
        "console_scripts": [
            "dp-launching = dp.launching.cli.commands.launching:main",
        ]
    },
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[],
    python_requires=">=3.8",
    install_requires=[
        "fastapi",
        "uvicorn",
        "wcmatch",
        "pydantic>=1.10.5",
        "pydantic_cli>=4.3.0",
        "Deprecated",
        "dict-deep>=4.1.2",
    ],
)



from .basic import *  # noqa
from .benchmark import *  # noqa
from .bio import *  # noqa
from .bohrium import *  # noqa
from .datahub import *  # noqa
from .dflow import *  # noqa
from .io import *  # noqa
from .dataset import *  # noqa
